import pytest
import numpy as np
from ..skySurvey import SkySurvey

# Set up the random number generator
np.random.seed(1234)


def test_basic():
    """
    basic test of equivalent width
    """
    survey = SkySurvey()
    import astropy.units as u
    survey["INTEN"] = survey["INTEN"].data * u.R

    ha_wave = 656.3 * u.nm
    optical_ha_equiv = u.doppler_optical(ha_wave)
    dv = (survey["VELOCITY"][0][1] - survey["VELOCITY"][0][0]) * u.km/u.s
    dlambda = (dv).to(u.AA, equivalencies=optical_ha_equiv) - \
        (0 * u.km/u.s).to(u.AA, equivalencies=optical_ha_equiv)

    survey["BKG"] = np.abs(np.random.randn(len(survey["INTEN"]))) * 100.
    survey["BKGSD"] = survey["BKG"] * .1


    ew, ew_error = survey.get_equivalent_width(continuum = survey["BKG"] / 22.8 * u.R / dlambda, 
        continuum_error = survey["BKGSD"] / 22.8 * u.R / dlambda, return_sigma = True)

    ew2, ew2_error = survey.get_equivalent_width(intensity = survey["INTEN"].data * u.R, 
        intensity_error = survey["ERROR"].data * u.R)

    assert np.allclose(ew.value, ew2.value)

def test_no_continuum_error():
    """
    basic test of equivalent width logs warning with no cont_error
    """
    survey = SkySurvey()
    import astropy.units as u
    survey["INTEN"] = survey["INTEN"].data * u.R

    ha_wave = 656.3 * u.nm
    optical_ha_equiv = u.doppler_optical(ha_wave)
    dv = (survey["VELOCITY"][0][1] - survey["VELOCITY"][0][0]) * u.km/u.s
    dlambda = (dv).to(u.AA, equivalencies=optical_ha_equiv) - \
        (0 * u.km/u.s).to(u.AA, equivalencies=optical_ha_equiv)

    survey["BKG"] = np.abs(np.random.randn(len(survey["INTEN"]))) * 100.


    ew, ew_error = survey.get_equivalent_width(return_sigma = True)

    ew2, ew2_error = survey.get_equivalent_width(intensity = survey["INTEN"].data * u.R, 
        intensity_error = survey["ERROR"].data * u.R)

    assert np.allclose(ew.value, ew2.value)


def test_continuum_unit_error():
    """
    continuum must have units
    """
    survey = SkySurvey()
    try:
        survey.get_equivalent_width(continuum = 5)
    except TypeError:
        assert True
    else:
        assert False

def test_intensity_error():
    """
    test intensity error providing
    """
    survey = SkySurvey()
    import astropy.units as u
    survey["INTEN"] = survey["INTEN"].data * u.R

    ha_wave = 656.3 * u.nm
    optical_ha_equiv = u.doppler_optical(ha_wave)
    dv = (survey["VELOCITY"][0][1] - survey["VELOCITY"][0][0]) * u.km/u.s
    dlambda = (dv).to(u.AA, equivalencies=optical_ha_equiv) - \
        (0 * u.km/u.s).to(u.AA, equivalencies=optical_ha_equiv)

    survey["BKG"] = np.abs(np.random.randn(len(survey["INTEN"]))) * 100.
    survey["BKGSD"] = survey["BKG"] * .1
    import astropy.units as u
    error = survey["ERROR"].data * u.R

    ew, ew_error = survey.get_equivalent_width(intensity_error = error)
    ew2, ew_error2 = survey.get_equivalent_width()

    assert np.allclose(ew.value, ew2.value)

def test_intensity_error_with_intensity():
    """
    test intensity error not provided but intensity provided
    """
    survey = SkySurvey()
    import astropy.units as u
    survey["INTEN"] = survey["INTEN"].data * u.R

    ha_wave = 656.3 * u.nm
    optical_ha_equiv = u.doppler_optical(ha_wave)
    dv = (survey["VELOCITY"][0][1] - survey["VELOCITY"][0][0]) * u.km/u.s
    dlambda = (dv).to(u.AA, equivalencies=optical_ha_equiv) - \
        (0 * u.km/u.s).to(u.AA, equivalencies=optical_ha_equiv)

    survey["BKG"] = np.abs(np.random.randn(len(survey["INTEN"]))) * 100.
    survey["BKGSD"] = survey["BKG"] * .1
    try:
        survey.get_equivalent_width(intensity = survey["INTEN"])
    except TypeError:
        assert True
    else:
        assert False

def test_no_sigma():
    """
    test return sigma
    """
    survey = SkySurvey()
    import astropy.units as u
    survey["INTEN"] = survey["INTEN"].data * u.R

    ha_wave = 656.3 * u.nm
    optical_ha_equiv = u.doppler_optical(ha_wave)
    dv = (survey["VELOCITY"][0][1] - survey["VELOCITY"][0][0]) * u.km/u.s
    dlambda = (dv).to(u.AA, equivalencies=optical_ha_equiv) - \
        (0 * u.km/u.s).to(u.AA, equivalencies=optical_ha_equiv)

    survey["BKG"] = np.abs(np.random.randn(len(survey["INTEN"]))) * 100.
    survey["BKGSD"] = survey["BKG"] * .1
    ew = survey.get_equivalent_width(return_sigma = False)
    ew2, ew_error2 = survey.get_equivalent_width()

    assert np.allclose(ew.value, ew2.value)




import pytest
import numpy as np
from ..skySurvey import SkySurvey

# Set up the random number generator
np.random.seed(1234)

# Load global survey
survey = SkySurvey()



def test_basic():
	"""
	basic test of spectral stacking
	"""
	stack_1 = survey[0:5].stack_spectra_bootstrap()
	stack_2 = survey[0:5].stack_spectra_bootstrap(set_name = "test")
	assert np.allclose(stack_1["DATA"], stack_2["DATA"], equal_nan = True, atol = 1e-4)

def test_columns():
	"""
	column test of spectral stacking
	"""
	stack_1 = survey[0:5].stack_spectra_bootstrap(data_column = "DATA")
	stack_2 = survey[0:5].stack_spectra_bootstrap(variance_column = "VARIANCE")
	assert np.allclose(stack_1["DATA"], stack_2["DATA"], equal_nan = True, atol = 1e-4)


def test_velocity_name():
	"""
	velocity and name input test of spectral stacking
	"""
	stack_1 = survey[0:5].stack_spectra_bootstrap(velocity = np.round(survey[0]["VELOCITY"]))
	stack_2 = survey[0:5].stack_spectra_bootstrap(set_name = "Test")
	assert np.allclose(stack_1["DATA"], stack_2["DATA"], equal_nan = True, atol = 1e-4)

def test_boot():
	"""
	bootstrap parameter input test of spectral stacking
	"""
	stack_1 = survey[0:5].stack_spectra_bootstrap(n_boot = 10000)
	stack_2 = survey[0:5].stack_spectra_bootstrap(ci = 95)
	assert np.allclose(stack_1["DATA"], stack_2["DATA"], equal_nan = True, atol = 1e-4)	

def test_estimator():
	"""
	estimator input test of spectral stacking
	"""
	stack_1 = survey[0:5].stack_spectra_bootstrap(estimator = np.nanmean)
	stack_2 = survey[0:5].stack_spectra_bootstrap()
	assert np.allclose(stack_1["DATA"], stack_2["DATA"], equal_nan = True, atol = 1e-4)	

def test_window_combine_basic():
	"""
	test combining windows
	"""	

	from ..spectralStack import combine_velocity_windows_bootstrap as cvwb
	window_1 = survey[10:20]
	window_2 = survey[30:40]
	window_1["VELOCITY"] += 50
	window_2["VELOCITY"] -= 50
	window_single = survey[50]

	stack_1 = cvwb([window_1, window_2], survey_spectra = window_single)
	stack_2 = cvwb([window_2, window_1], survey_spectra = window_single, set_name = "test")
	works = np.allclose(stack_1["DATA"], stack_2["DATA"], equal_nan = True, atol = 1e-2)
	print(works)
	assert works

